@@ -140,7 +140,10 @@ class DSObject(object):
 
     def __init__(self, object_id, metadata=None, file_path=None):
         self._update_signal_match = None
+        self._object_id = None
+
         self.set_object_id(object_id)
+
         self._metadata = metadata
         self._file_path = file_path
         self._destroyed = False
@@ -250,9 +253,11 @@ class RawObject(object):
         # to create hardlinks to jobject files
         # and w/o this, it wouldn't work since we have file from mounted device
         if self._file_path is None:
+            data_path = os.path.join(env.get_profile_path(), 'data')
             self._file_path = tempfile.mktemp(
-                    prefix='rawobject',
-                    dir=os.path.join(env.get_profile_path(), 'data'))
+                    prefix='rawobject', dir=data_path)
+            if not os.path.exists(data_path):
+                os.makedirs(data_path)
             os.symlink(self.object_id, self._file_path)
         return self._file_path
 
