@@ -53,8 +53,19 @@ class Profile(object):
         self._pubkey = None
         self._privkey_hash = None
 
-        self.pubkey = self._load_pubkey()
-        self.privkey_hash = self._hash_private_key()
+    def _get_pubkey(self):
+        if self._pubkey is None:
+            self._pubkey = self._load_pubkey()
+        return self._pubkey
+
+    pubkey = property(fget=_get_pubkey)
+
+    def _get_privkey_hash(self):
+        if self._privkey_hash is None:
+            self._privkey_hash = self._hash_private_key()
+        return self._privkey_hash
+
+    privkey_hash = property(fget=_get_privkey_hash)
 
     def is_valid(self):
         client = gconf.client_get_default()
@@ -184,12 +195,16 @@ class Profile(object):
             '# log files and features\n'\
             '#export LM_DEBUG=net\n' \
             '#export GABBLE_DEBUG=all\n' \
-            '#export ' \
-            'GABBLE_LOGFILE=$HOME/.sugar/default/logs/telepathy-gabble.log\n' \
+            '#export GABBLE_LOGFILE=' \
+            '$HOME/.sugar/$SUGAR_PROFILE/logs/telepathy-gabble.log\n' \
             '#export SALUT_DEBUG=all\n' \
-            '#export ' \
-            'SALUT_LOGFILE=$HOME/.sugar/default/logs/telepathy-salut.log\n' \
+            '#export SALUT_LOGFILE=' \
+            '$HOME/.sugar/$SUGAR_PROFILE/logs/telepathy-salut.log\n' \
             '#export GIBBER_DEBUG=all\n' \
+            '#export WOCKY_DEBUG=all\n' \
+            '#export MC_LOGFILE=' \
+            '$HOME/.sugar/$SUGAR_PROFILE/logs/mission-control.log\n' \
+            '#export MC_DEBUG=all\n' \
             '#export PRESENCESERVICE_DEBUG=1\n' \
             '#export SUGAR_LOGGER_LEVEL=debug\n\n' \
             '# Uncomment the following line to enable core dumps\n' \
